
from wagtail_jsonschema_forms.jsonschema_generator import JsonSchemaGenerator


class TestJsonschemaGenerator:

    def test_singleline_generator(self, singleline, jsonschema_singleline):
        assert JsonSchemaGenerator.generate_singleline(singleline) == jsonschema_singleline

    def test_multiline_generator(self, multiline, jsonschema_multiline):
        assert JsonSchemaGenerator.generate_multiline(multiline) == jsonschema_multiline

    def test_date_generator(self, date, jsonschema_date):
        assert JsonSchemaGenerator.generate_date(date) == jsonschema_date

    def test_datetime_generator(self, datetime, jsonschema_datetime):
        assert JsonSchemaGenerator.generate_datetime(datetime) == jsonschema_datetime

    def test_email_generator(self, email, jsonschema_email):
        assert JsonSchemaGenerator.generate_email(email) == jsonschema_email

    def test_url_generator(self, url, jsonschema_url):
        assert JsonSchemaGenerator.generate_url(url) == jsonschema_url

    def test_number_generator(self, number, jsonschema_number):
        assert JsonSchemaGenerator.generate_number(number) == jsonschema_number

    def test_dropdown_generator(self, dropdown, jsonschema_dropdown):
        assert JsonSchemaGenerator.generate_dropdown(dropdown) == jsonschema_dropdown

    def test_checkbox_generator(self, checkbox, jsonschema_checkbox):
        assert JsonSchemaGenerator.generate_checkbox(checkbox) == jsonschema_checkbox

    def test_schemajson_generator(self, wagtail_form, jsonschema_generated):
        assert JsonSchemaGenerator.wagtail_form_to_json_schema(wagtail_form) == jsonschema_generated



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = 'changemepliz'
DEBUG = True
ALLOWED_HOSTS = "*"

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'wagtail.wagtailsites',
    'wagtail.wagtailforms',
    'wagtail.wagtailusers',
    'wagtail.wagtaildocs',
    'wagtail.wagtailimages',
    'wagtail.wagtailcore',
    'wagtail.contrib.modeladmin',
    'wagtail.tests.testapp',
    'taggit',
    'wagtail_jsonschema_forms'
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'tests.testapp.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'tests/testapp/wagtail_jsonschema_forms.db',
        'USER': '',
        'PASSWORD': '',
    }
}

WAGTAIL_SITE_NAME = 'Wagtail JsonSchema Test Site'



from collections import OrderedDict
import json


class JsonSchemaGenerator:

    FIELD_TYPES = {
        "singleline": "generate_singleline",
        "multiline": "generate_multiline",
        "date": "generate_date",
        "datetime": "generate_datetime",
        "email": "generate_email",
        "url": "generate_url",
        "number": "generate_number",
        "dropdown": "generate_dropdown",
        "checkbox": "generate_checkbox",
    }

    @staticmethod
    def generate_singleline(field):
        return {
            "type": "string",
            "description": field.help_text,
            "maxLength": 150,
            "default": field.default_value
        }

    @staticmethod
    def generate_multiline(field):
        return {
            "type": "string",
            "description": field.help_text,
            "maxLength": 500,
            "default": field.default_value
        }

    @staticmethod
    def generate_email(field):
        return {
            "type": "string",
            "description": field.help_text,
            "maxLength": 100,
            "format": "email",
            "default": field.default_value
        }

    @staticmethod
    def generate_date(field):
        return {
            "type": "string",
            "description": field.help_text,
            "format": "date",
            "default": field.default_value
        }

    @staticmethod
    def generate_datetime(field):
        return {
            "type": "string",
            "description": field.help_text,
            "format": "date-time",
            "default": field.default_value
        }

    @staticmethod
    def generate_url(field):
        return {
            "type": "string",
            "description": field.help_text,
            "maxLength": 100,
            "format": "uri",
            "default": field.default_value
        }

    @staticmethod
    def generate_number(field):
        return {
            "type": "number",
            "description": field.help_text,
            "default": field.default_value
        }

    @staticmethod
    def generate_dropdown(field):
        return {
            "type": "string",
            "description": field.help_text,
            "default": field.default_value,
            "enum": field.choices.split(",")
        }

    @staticmethod
    def generate_checkbox(field):
        return {
            "type": "boolean",
            "description": field.help_text,
            "default": field.default_value,
            "format": "checkbox"
        }

    @staticmethod
    def wagtail_form_to_json_schema(form):
        """
        This function expects a wagtail form and return a Json Schema representation
        :param form: wagtail form
        :return: dict
        """
        json_schema = OrderedDict()
        json_schema["title"] = form.slug
        json_schema["type"] = "object"

        properties = OrderedDict()
        required = []
        for field in form.form_fields.all():
            field_type = field.field_type
            if field_type in JsonSchemaGenerator.FIELD_TYPES:
                properties[field.label] = getattr(JsonSchemaGenerator,
                                                  JsonSchemaGenerator.FIELD_TYPES[field_type])(field)

                if field.required:
                    required.append(field.label)

        json_schema["properties"] = properties
        json_schema["required"] = required
        return json.dumps(json_schema)



import json
from collections import OrderedDict
import pytest
from model_mommy import mommy
from wagtail.tests.testapp.models import FormPage, FormField
from wagtail.wagtailcore.models import Page


@pytest.mark.django_db()
@pytest.fixture()
def form_page(admin_user):
    root_page = Page.objects.first()
    form = FormPage()
    form.title = "form"
    form.slug = "form"
    form.live = True
    form.owner = admin_user
    root_page.add_child(instance=form)
    form.save()
    return form


@pytest.mark.django_db()
@pytest.fixture()
def singleline(form_page):
    return mommy.make(FormField, page=form_page, field_type="singleline", label="singleline",
                      help_text="singleline", default_value="singleline", required=True)


@pytest.fixture()
def jsonschema_singleline():
    return {
        "type": "string",
        "description": "singleline",
        "maxLength": 150,
        "default": "singleline"
    }


@pytest.mark.django_db()
@pytest.fixture()
def multiline(form_page):
    return mommy.make(FormField, page=form_page, field_type="multiline", label="multiline", default_value="multiline",
                      required=False)


@pytest.fixture()
def jsonschema_multiline():
    return {
        "type": "string",
        "description": "",
        "maxLength": 500,
        "default": "multiline"
    }


@pytest.mark.django_db()
@pytest.fixture()
def date(form_page):
    return mommy.make(FormField, page=form_page, field_type="date", label="date", help_text="date", required=True)


@pytest.fixture()
def jsonschema_date():
    return {
        "type": "string",
        "description": "date",
        "format": "date",
        "default": ""
    }


@pytest.mark.django_db()
@pytest.fixture()
def datetime(form_page):
    return mommy.make(FormField, page=form_page, field_type="datetime", label="datetime", help_text="datetime",
                      required=False)


@pytest.fixture()
def jsonschema_datetime():
    return {
        "type": "string",
        "description": "datetime",
        "format": "date-time",
        "default": ""
    }


@pytest.mark.django_db()
@pytest.fixture()
def email(form_page):
    return mommy.make(FormField, page=form_page, field_type="email", label="email", default_value="email",
                      required=False)


@pytest.fixture()
def jsonschema_email():
    return {
        "type": "string",
        "description": "",
        "maxLength": 100,
        "format": "email",
        "default": "email"
    }


@pytest.mark.django_db()
@pytest.fixture()
def url(form_page):
    return mommy.make(FormField, page=form_page, field_type="url", label="url", required=False)


@pytest.fixture()
def jsonschema_url():
    return {
        "type": "string",
        "description": "",
        "maxLength": 100,
        "format": "uri",
        "default": ""
    }


@pytest.mark.django_db()
@pytest.fixture()
def number(form_page):
    return mommy.make(FormField, page=form_page, field_type="number", label="number", required=True)


@pytest.fixture()
def jsonschema_number():
    return {
        "type": "number",
        "description": "",
        "default": ""
    }


@pytest.mark.django_db()
@pytest.fixture()
def dropdown(form_page):
    return mommy.make(FormField, page=form_page, field_type="dropdown", label="dropdown", help_text="dropdown",
                      choices="dropdown,dropdown2,dropdown3", required=False)


@pytest.fixture()
def jsonschema_dropdown():
    return {
        "type": "string",
        "description": "dropdown",
        "default": "",
        "enum": ["dropdown", "dropdown2", "dropdown3"]
    }


@pytest.mark.django_db()
@pytest.fixture()
def checkbox(form_page):
    return mommy.make(FormField, page=form_page, field_type="checkbox", label="checkbox", required=True)


@pytest.fixture()
def jsonschema_checkbox():
    return {
        "type": "boolean",
        "description": "",
        "default": "",
        "format": "checkbox"
    }


@pytest.mark.django_db()
@pytest.fixture()
def wagtail_form(checkbox, dropdown, number, url, email, datetime, date, multiline, singleline):
    return singleline.page


@pytest.fixture()
def jsonschema_generated(jsonschema_singleline, jsonschema_multiline, jsonschema_date, jsonschema_datetime,
                         jsonschema_email, jsonschema_url, jsonschema_number, jsonschema_dropdown, jsonschema_checkbox):

    properties = OrderedDict()
    properties["checkbox"] = jsonschema_checkbox
    properties["dropdown"] = jsonschema_dropdown
    properties["number"] = jsonschema_number
    properties["url"] = jsonschema_url
    properties["email"] = jsonschema_email
    properties["datetime"] = jsonschema_datetime
    properties["date"] = jsonschema_date
    properties["multiline"] = jsonschema_multiline
    properties["singleline"] = jsonschema_singleline

    json_schema = OrderedDict()
    json_schema["title"] = "form"
    json_schema["type"] = "object"
    json_schema["properties"] = properties
    json_schema["required"] = ["checkbox", "number", "date", "singleline"]
    return json.dumps(json_schema)


########################################################################################################################
# Parser fixtures
########################################################################################################################

@pytest.mark.django_db()
@pytest.fixture()
def single_line(form_page):
    return mommy.make(FormField, page=form_page, field_type="singleline", label="single line",
                      help_text="singleline", default_value="singleline", required=True)


@pytest.mark.django_db()
@pytest.fixture()
def simple_wagtail_form(single_line):
    return single_line.page


@pytest.fixture()
def simple_form_json_correct():
    return {"single line": "Test value"}


@pytest.fixture()
def simple_form_json_incorrect():
    return {"single line": 31351351}


@pytest.fixture()
def json_submited():
    return {"single line": "Test value"}


@pytest.fixture()
def json_ready_to_save():
    return '{"single-line": "Test value"}'




from django.core.serializers.json import DjangoJSONEncoder
from django.utils.six import text_type
from django.utils.text import slugify

import json
from jsonschema.validators import validator_for
from unidecode import unidecode


class JsonSchemaParser:

    @staticmethod
    def validate_json(json_form, schema, *args, **kwargs):
        """
        Function that validates the json recieved under the schema and returns a dict with the errors, if there are any
        :param json_form: dict
        :param schema: dict
        :return dict
        """
        cls = validator_for(schema)
        cls.check_schema(schema)
        json_errors = cls(schema, *args, **kwargs).iter_errors(json_form)
        errors = {}
        for error in json_errors:
            # errors can be from fields validation or required camps
            form_field = error.path[0] if error.path else error.validator_value[0]
            errors[form_field] = error.message
        return errors

    @staticmethod
    def process_form_submission(form, json_submited):
        """
        Function that accepts form instance with submitted data and page and creates submission instance.
        :param form: Form Page
        :param json_submited: str
        """
        form.get_submission_class().objects.create(form_data=JsonSchemaParser.prepare_form_data(json_submited),
                                                   page=form)

    @staticmethod
    def prepare_form_data(json_submited):
        """
        Function that prepare json field keys to the submition
        :param json_submited: dict
        :return str
        """
        form_data = {}
        for field_key, value in json_submited.items():
            # same conversion as clean_name in wagtail.wagtailforms.models.AbstractFormField
            form_data[str(slugify(text_type(unidecode(field_key))))] = value
        return json.dumps(form_data, cls=DjangoJSONEncoder)



from django.db import models
from wagtail.wagtailforms.models import AbstractForm, AbstractEmailForm

from wagtail_jsonschema_forms.mixins import JsonSchemaFormMixin, JsonSchemaEmailFormMixin


class AbstractJsonSchemaForm(JsonSchemaFormMixin, AbstractForm):
    """
    A Form Page that generates a JSON Schema representation of the fields and save it to the model.
    Pages implementing a form that need to generate JSON Schema should inherit from it
    """

    json_schema = models.TextField(help_text=u"Form json Schema representation", blank=True)

    class Meta:
        abstract = True


class AbstractJsonSchemaEmailForm(JsonSchemaEmailFormMixin, AbstractEmailForm):
    """
    A Form Page that generates a JSON Schema representation of the fields and save it to the model. It also send email.
    Pages implementing a form that need to generate JSON Schema and need to send an email should inherit from it
    """

    json_schema = models.TextField(help_text=u"Form json Schema representation", blank=True)

    class Meta:
        abstract = True




from wagtail.wagtailadmin.utils import send_mail
from wagtail_jsonschema_forms.jsonschema_generator import JsonSchemaGenerator
from wagtail_jsonschema_forms.jsonschema_parser import JsonSchemaParser


class JsonSchemaFormMixin:

    def save(self, *args, **kwargs):
        self.json_schema = self.generate_json_schema()
        super().save(*args, **kwargs)

    def generate_json_schema(self):
        """
        Function that generates the json schema representation of the form
        """
        return JsonSchemaGenerator.wagtail_form_to_json_schema(self)

    def process_form_submission(self, form):
        """
        Function that check the way that the form has been submited
        :param form: WagtailForm or dict
        :return:
        """
        if hasattr(form, "data"):
            super().process_form_submission(form)
        else:
            self.process_json_form_submission(form)

    def process_json_form_submission(self, json_submited):
        """
        Function that process json form submission
        :param json_submited: dict
        :return:
        """
        JsonSchemaParser.process_form_submission(self, json_submited)


class JsonSchemaEmailFormMixin(JsonSchemaFormMixin):

    def process_json_form_submission(self, json_submited):
        """
        Function that process json form submission and send an email from a json representation of the submitted data
        :param json_submited: dict
        :return:
        """
        JsonSchemaParser.process_form_submission(self, json_submited)
        self.send_mail_from_json(json_submited)

    def send_mail_from_json(self, form_fields):
        """
        Function that send a email from a json representation of the post data of the form.
        Based on wagtail.wagtailforms.models.AbstractEmailForm.send_mail
        :param form_fields: dict
        :return:
        """
        if self.to_address:
            addresses = [x.strip() for x in self.to_address.split(',')]
            content = []
            for field_key, value in form_fields.items():
                if isinstance(value, list):
                    value = ', '.join(value)
                content.append('{}: {}'.format(field_key.capitalize(), value))
            content = '\n'.join(content)
            send_mail(self.subject, content, addresses, self.from_address)




import json

from wagtail_jsonschema_forms.jsonschema_parser import JsonSchemaParser
from wagtail_jsonschema_forms.jsonschema_generator import JsonSchemaGenerator


class TestJsonschemaParser:

    def test_validate_json_correct(self, simple_form_json_correct, simple_wagtail_form):
        form_schema = JsonSchemaGenerator.wagtail_form_to_json_schema(simple_wagtail_form)
        form_schema = json.loads(form_schema)
        errors = JsonSchemaParser.validate_json(simple_form_json_correct, form_schema)
        assert errors == {}

    def test_validate_json_incorrect(self, simple_form_json_incorrect, simple_wagtail_form):
        form_schema = JsonSchemaGenerator.wagtail_form_to_json_schema(simple_wagtail_form)
        form_schema = json.loads(form_schema)
        errors = JsonSchemaParser.validate_json(simple_form_json_incorrect, form_schema)
        assert "single line" in errors

    def test_prepare_form_data(self, json_submited, json_ready_to_save):
        assert JsonSchemaParser.prepare_form_data(json_submited) == json_ready_to_save

    def test_process_form_submission(self, simple_wagtail_form, json_submited):
        JsonSchemaParser.process_form_submission(simple_wagtail_form, json_submited)
        assert simple_wagtail_form.get_submission_class().objects.all()
